@@ -42,7 +42,6 @@
 #include "rhythmdb.h"
 #include "rb-shell.h"
 #include "rb-daap-source.h"
-#include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
@@ -53,6 +52,8 @@
 #include "rb-display-page.h"
 #include "rb-builder-helpers.h"
 #include "rb-application.h"
+#include "rb-task-list.h"
+#include "rb-task-progress-simple.h"
 
 #include "rb-daap-plugin.h"
 
@@ -75,7 +76,6 @@ static void rb_daap_source_get_property  (GObject *object,
 				 	  GParamSpec *pspec);
 
 static void rb_daap_source_selected (RBDisplayPage *page);
-static void rb_daap_source_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
 static void disconnect_action_cb (GSimpleAction *, GVariant *, gpointer);
 
 static void rb_daap_entry_type_class_init (RBDAAPEntryTypeClass *klass);
@@ -93,8 +93,7 @@ struct RBDAAPSourcePrivate
 
 	GSList *playlist_sources;
 
-	const char *connection_status;
-	float connection_progress;
+	RBTaskProgress *connection_status;
 
 	gboolean tried_password;
 	gboolean disconnecting;
@@ -149,6 +148,7 @@ rb_daap_source_dispose (GObject *object)
 
 	/* we should already have been disconnected */
 	g_assert (source->priv->connection == NULL);
+	g_clear_object (&source->priv->connection_status);
 
 	G_OBJECT_CLASS (rb_daap_source_parent_class)->dispose (object);
 }
@@ -179,7 +179,6 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	object_class->set_property = rb_daap_source_set_property;
 
 	page_class->selected = rb_daap_source_selected;
-	page_class->get_status = rb_daap_source_get_status;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
@@ -325,7 +324,7 @@ rb_daap_source_new (RBShell *shell,
 {
 	RBSource *source;
 	RhythmDBEntryType *entry_type;
-	GdkPixbuf *icon;
+	GIcon *icon;
 	RhythmDB *db;
 	char *entry_type_name;
 	GSettings *settings;
@@ -358,7 +357,7 @@ rb_daap_source_new (RBShell *shell,
 					  "host", host,
 					  "port", port,
 					  "entry-type", entry_type,
-					  "pixbuf", icon,
+					  "icon", icon,
 					  "shell", shell,
 					  "visibility", TRUE,
 					  "password-protected", password_protected,
@@ -369,10 +368,7 @@ rb_daap_source_new (RBShell *shell,
 					  NULL));
 	g_object_unref (settings);
 	g_object_unref (builder);
-
-	if (icon != NULL) {
-		g_object_unref (icon);
-	}
+	g_object_unref (icon);
 
 	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 
@@ -526,8 +522,8 @@ connection_connecting_cb (DMAPConnection       *connection,
 			  float		        progress,
 			  RBDAAPSource         *source)
 {
-	GdkPixbuf *icon;
-	gboolean   is_connected;
+	GIcon *icon;
+	gboolean is_connected;
 	GObject *plugin;
 
 	rb_debug ("DAAP connection status: %d/%f", state, progress);
@@ -535,7 +531,6 @@ connection_connecting_cb (DMAPConnection       *connection,
 	switch (state) {
 	case DMAP_GET_INFO:
 	case DMAP_LOGIN:
-		source->priv->connection_status = _("Connecting to music share");
 		break;
 	case DMAP_GET_REVISION_NUMBER:
 		g_object_set (source, "load-status", RB_SOURCE_LOAD_STATUS_LOADING, NULL);
@@ -543,17 +538,18 @@ connection_connecting_cb (DMAPConnection       *connection,
 	case DMAP_GET_SONGS:
 	case DMAP_GET_PLAYLISTS:
 	case DMAP_GET_PLAYLIST_ENTRIES:
-		source->priv->connection_status = _("Retrieving songs from music share");
+		g_object_set (source->priv->connection_status,
+			      "task-label", _("Retrieving songs from music share"),
+			      "task-progress", progress,
+			      NULL);
 		break;
 	case DMAP_DONE:
 		g_object_set (source, "load-status", RB_SOURCE_LOAD_STATUS_LOADED, NULL);
+		g_object_set (source->priv->connection_status, "task-outcome", RB_TASK_OUTCOME_COMPLETE, NULL);
 	case DMAP_LOGOUT:
-		source->priv->connection_status = NULL;
 		break;
 	}
 
-	source->priv->connection_progress = progress;
-
 	rb_display_page_notify_status_changed (RB_DISPLAY_PAGE (source));
 
 	is_connected = dmap_connection_is_connected (DMAP_CONNECTION (connection));
@@ -564,11 +560,8 @@ connection_connecting_cb (DMAPConnection       *connection,
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin),
 					source->priv->password_protected,
 					is_connected);
-	g_object_set (source, "pixbuf", icon, NULL);
-	if (icon != NULL) {
-		g_object_unref (icon);
-	}
-
+	g_object_set (source, "icon", icon, NULL);
+	g_clear_object (&icon);
 	g_object_unref (plugin);
 }
 
@@ -576,8 +569,8 @@ static void
 connection_disconnected_cb (DMAPConnection   *connection,
 			    RBDAAPSource     *source)
 {
-	GdkPixbuf *icon;
-	GObject   *plugin;
+	GIcon *icon;
+	GObject *plugin;
 
 	rb_debug ("DAAP connection disconnected");
 
@@ -589,10 +582,8 @@ connection_disconnected_cb (DMAPConnection   *connection,
 		icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin),
 						source->priv->password_protected,
 						FALSE);
-		g_object_set (source, "pixbuf", icon, NULL);
-		if (icon != NULL) {
-			g_object_unref (icon);
-		}
+		g_object_set (source, "icon", icon, NULL);
+		g_clear_object (&icon);
 	}
 
 	g_object_unref (plugin);
@@ -621,6 +612,8 @@ rb_daap_source_connection_cb (DMAPConnection   *connection,
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 	RBShell *shell = NULL;
+	GSettings *plugin_settings;
+	GSettings *settings;
 	GSList *playlists;
 	GSList *l;
 	RhythmDBEntryType *entry_type;
@@ -644,21 +637,15 @@ rb_daap_source_connection_cb (DMAPConnection   *connection,
 	g_object_get (daap_source,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
+		      "settings", &plugin_settings,
 		      NULL);
+	settings = g_settings_get_child (plugin_settings, "source");
 	playlists = dmap_connection_get_playlists (DMAP_CONNECTION (daap_source->priv->connection));
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
 		DMAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
-		char *settings_name;
 
-		/* Construct a unique settings name for this playlist, as <Share Name>_<Playlist> */
-		/* XXX disabled for now, not sure it's a good idea */
-		/*settings_name = g_strjoin (NULL, daap_source->priv->service_name, "_", playlist->name, NULL); */
-
-		settings_name = "/org/gnome/rhythmbox/plugins/daap/source";
-
-		playlist_source = rb_static_playlist_source_new (shell, playlist->name, settings_name, FALSE, entry_type);
-		/*g_free (settings_name);*/
+		playlist_source = rb_static_playlist_source_new (shell, playlist->name, settings, FALSE, entry_type);
 
 		g_list_foreach (playlist->uris, (GFunc)_add_location_to_playlist, playlist_source);
 
@@ -666,6 +653,8 @@ rb_daap_source_connection_cb (DMAPConnection   *connection,
 		daap_source->priv->playlist_sources = g_slist_prepend (daap_source->priv->playlist_sources, playlist_source);
 	}
 
+	g_object_unref (settings);
+	g_object_unref (plugin_settings);
 	g_object_unref (shell);
 	g_object_unref (entry_type);
 }
@@ -680,6 +669,7 @@ rb_daap_source_selected (RBDisplayPage *page)
 	DMAPDb *db = NULL;
 	char *name = NULL;
 	RhythmDBEntryType *entry_type;
+	RBTaskList *tasklist;
 
 	RB_DISPLAY_PAGE_CLASS (rb_daap_source_parent_class)->selected (page);
 
@@ -692,11 +682,22 @@ rb_daap_source_selected (RBDisplayPage *page)
 		      "name", &name,
 		      "entry-type", &entry_type,
 		      NULL);
-	g_object_get (shell, "db", &rdb, NULL);
+	g_object_get (shell,
+		      "db", &rdb,
+		      "task-list", &tasklist,
+		      NULL);
 	db = DMAP_DB (rb_rhythmdb_dmap_db_adapter_new (rdb, entry_type));
 
 	factory = DMAP_RECORD_FACTORY (rb_daap_record_factory_new ());
 
+	daap_source->priv->connection_status = rb_task_progress_simple_new ();
+	g_object_set (daap_source->priv->connection_status,
+		      "task-label", _("Connecting to music share"),
+		      "task-progress", -0.5,
+		      NULL);
+	rb_task_list_add_task (tasklist, RB_TASK_PROGRESS (daap_source->priv->connection_status));
+	g_object_unref (tasklist);
+
 	daap_source->priv->connection = daap_connection_new (name,
 							     daap_source->priv->host,
 							     daap_source->priv->port,
@@ -830,29 +831,6 @@ rb_daap_source_get_headers (RBDAAPSource *source,
 	return dmap_connection_get_headers (source->priv->connection, uri);
 }
 
-static void
-rb_daap_source_get_status (RBDisplayPage *page,
-			   char **text,
-			   char **progress_text,
-			   float *progress)
-{
-	RBDAAPSource *daap_source = RB_DAAP_SOURCE (page);
-
-	if (daap_source->priv->connection_status != NULL) {
-		if (text != NULL) {
-			*text = g_strdup (daap_source->priv->connection_status);
-		}
-
-		if (progress != NULL) {
-			*progress = daap_source->priv->connection_progress;
-		}
-
-		return;
-	}
-
-	RB_DISPLAY_PAGE_CLASS (rb_daap_source_parent_class)->get_status (page, text, progress_text, progress);
-}
-
 void
 _rb_daap_source_register_type (GTypeModule *module)
 {
