@@ -49,7 +49,6 @@
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-shell-player.h"
-#include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
 #include "rb-util.h"
@@ -908,7 +907,7 @@ episode_activated_cb (RBEntryView *view,
 static void
 podcast_entry_changed_cb (RhythmDB *db,
 			  RhythmDBEntry *entry,
-			  GArray *changes,
+			  GPtrArray *changes,
 			  RBPodcastSource *source)
 {
 	RhythmDBEntryType *entry_type;
@@ -921,8 +920,7 @@ podcast_entry_changed_cb (RhythmDB *db,
 
 	feed_changed = FALSE;
 	for (i = 0; i < changes->len; i++) {
-		GValue *v = &g_array_index (changes, GValue, i);
-		RhythmDBEntryChange *change = g_value_get_boxed (v);
+		RhythmDBEntryChange *change = g_ptr_array_index (changes, i);
 
 		switch (change->prop) {
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
@@ -1026,25 +1024,10 @@ rb_podcast_source_new (RBShell *shell,
 					  "settings", g_settings_get_child (settings, "source"),
 					  "toolbar-menu", toolbar,
 					  NULL));
+	rb_display_page_set_icon_name (RB_DISPLAY_PAGE (source), icon_name);
 	g_object_unref (settings);
 	g_object_unref (builder);
 
-	if (icon_name != NULL) {
-		GdkPixbuf *pixbuf;
-		gint size;
-
-		gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   icon_name,
-						   size,
-						   0, NULL);
-
-		if (pixbuf != NULL) {
-			g_object_set (source, "pixbuf", pixbuf, NULL);
-			g_object_unref (pixbuf);
-		}
-	}
-
 	return source;
 }
 
