@@ -37,7 +37,6 @@
 #include <widgets/rb-search-entry.h>
 #include <shell/rb-shell-preferences.h>
 #include <shell/rb-track-transfer-batch.h>
-#include <rhythmdb/rhythmdb-import-job.h>
 
 G_BEGIN_DECLS
 
@@ -59,8 +58,6 @@ typedef struct _RBSource	RBSource;
 typedef struct _RBSourceClass	RBSourceClass;
 typedef struct _RBSourcePrivate	RBSourcePrivate;
 
-typedef void (*RBSourceActionCallback) (GtkAction *action, RBSource *source);
-
 GType rb_source_eof_type_get_type (void);
 #define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
@@ -74,8 +71,6 @@ GType rb_source_load_status_get_type (void);
 #define RB_IS_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SOURCE))
 #define RB_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SOURCE, RBSourceClass))
 
-#define RB_SOURCE_ICON_SIZE	GTK_ICON_SIZE_LARGE_TOOLBAR
-
 typedef gboolean (*RBSourceFeatureFunc) (RBSource *source);
 typedef const char * (*RBSourceStringFunc) (RBSource *source);
 typedef void (*RBSourceAddCallback) (RBSource *source, const char *uri, gpointer data);
@@ -92,6 +87,7 @@ struct _RBSourceClass
 
 	/* signals */
 	void (*filter_changed)	(RBSource *source);
+	void (*reset_filters)	(RBSource *source);
 
 	/* methods */
 
@@ -101,7 +97,6 @@ struct _RBSourceClass
 	gboolean	(*impl_can_rename)	(RBSource *source);
 
 	void		(*impl_search)		(RBSource *source, RBSourceSearch *search, const char *cur_text, const char *new_text);
-	void		(*impl_reset_filters)	(RBSource *source);
 
 	gboolean	(*impl_can_cut)		(RBSource *source);
 	gboolean	(*impl_can_delete)	(RBSource *source);
@@ -132,8 +127,9 @@ struct _RBSourceClass
 
 	gboolean	(*impl_can_pause)	(RBSource *source);
 	RBSourceEOFType	(*impl_handle_eos)	(RBSource *source);
+	void		(*impl_get_playback_status) (RBSource *source, char **text, float *progress);
 
-	char *		(*impl_get_delete_action) (RBSource *source);
+	char *		(*impl_get_delete_label) (RBSource *source);
 };
 
 GType		rb_source_get_type		(void);
@@ -156,8 +152,6 @@ void		rb_source_search		(RBSource *source,
 						 const char *cur_text,
 						 const char *new_text);
 
-void		rb_source_reset_filters		(RBSource *source);
-
 gboolean	rb_source_can_cut		(RBSource *source);
 gboolean	rb_source_can_delete		(RBSource *source);
 gboolean	rb_source_can_move_to_trash	(RBSource *source);
@@ -189,26 +183,26 @@ void		rb_source_add_uri		(RBSource *source,
 gboolean	rb_source_can_pause		(RBSource *source);
 RBSourceEOFType	rb_source_handle_eos		(RBSource *source);
 
-char *		rb_source_get_delete_action	(RBSource *source);
+char *		rb_source_get_delete_label	(RBSource *source);
 
 GList *		rb_source_gather_selected_properties (RBSource *source, RhythmDBPropType prop);
 
 void            rb_source_set_hidden_when_empty (RBSource *source,
                                                  gboolean  hidden);
+void		rb_source_get_playback_status	(RBSource *source,
+						 char **text,
+						 float *progress);
 
 /* Protected methods, should only be used by objects inheriting from RBSource */
 
 gboolean	_rb_source_check_entry_type	(RBSource *source,
 						 RhythmDBEntry *entry);
 
-void		_rb_source_set_import_status	(RBSource *source,
-						 RhythmDBImportJob *job,
-						 char **progress_text,
-						 float *progress);
 void		rb_source_bind_settings		(RBSource *source,
 						 GtkWidget *entry_view,
 						 GtkWidget *paned,
 						 GtkWidget *browser);
+void		rb_source_notify_playback_status_changed (RBSource *source);
 
 G_END_DECLS
 
