@@ -54,7 +54,6 @@
 #include "rb-library-browser.h"
 #include "rb-util.h"
 #include "rb-debug.h"
-#include "rb-stock-icons.h"
 #include "rb-file-helpers.h"
 #include "rb-playlist-xml.h"
 #include "rb-source-search-basic.h"
@@ -147,8 +146,6 @@ typedef struct
 
 } RBStaticPlaylistSourcePrivate;
 
-static gpointer playlist_pixbuf = NULL;
-
 static void
 rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 {
@@ -188,30 +185,6 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	g_type_class_add_private (klass, sizeof (RBStaticPlaylistSourcePrivate));
 }
 
-static void
-set_playlist_pixbuf (RBStaticPlaylistSource *source)
-{
-	if (playlist_pixbuf == NULL) {
-		gint size;
-		gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
-		playlist_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-							    RB_STOCK_PLAYLIST,
-							    size,
-							    0, NULL);
-		if (playlist_pixbuf) {
-			g_object_add_weak_pointer (playlist_pixbuf,
-					   (gpointer *) &playlist_pixbuf);
-
-			g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
-
-			/* drop the initial reference to the icon */
-			g_object_unref (playlist_pixbuf);
-		}
-	} else {
-		g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
-	}
-}
-
 static void
 rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 {
@@ -269,7 +242,7 @@ rb_static_playlist_source_constructed (GObject *object)
 	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	psource = RB_PLAYLIST_SOURCE (source);
 
-	set_playlist_pixbuf (source);
+	rb_display_page_set_icon_name (RB_DISPLAY_PAGE (source), "folder-documents-symbolic");
 
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (priv->base_model, "show-hidden", TRUE, NULL);
@@ -297,6 +270,7 @@ rb_static_playlist_source_constructed (GObject *object)
 	}
 
 	gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
+	gtk_widget_set_no_show_all (GTK_WIDGET (priv->browser), TRUE);
 	g_signal_connect_object (priv->browser, "notify::output-model",
 				 G_CALLBACK (rb_static_playlist_source_browser_changed_cb),
 				 source, 0);
@@ -320,12 +294,14 @@ rb_static_playlist_source_constructed (GObject *object)
 
 	rb_source_search_basic_register (RHYTHMDB_PROP_SEARCH_MATCH, "search-match", _("Search all fields"));
 	rb_source_search_basic_register (RHYTHMDB_PROP_ARTIST_FOLDED, "artist", _("Search artists"));
+	rb_source_search_basic_register (RHYTHMDB_PROP_COMPOSER_FOLDED, "composer", _("Search composers"));
 	rb_source_search_basic_register (RHYTHMDB_PROP_ALBUM_FOLDED, "album", _("Search albums"));
 	rb_source_search_basic_register (RHYTHMDB_PROP_TITLE_FOLDED, "title", _("Search titles"));
 	
 	section = g_menu_new ();
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "search-match");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "artist");
+	rb_source_search_add_to_menu (section, "app", priv->search_action, "composer");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "album");
 	rb_source_search_add_to_menu (section, "app", priv->search_action, "title");
 
@@ -368,7 +344,7 @@ rb_static_playlist_source_constructed (GObject *object)
  * rb_static_playlist_source_new:
  * @shell: the #RBShell
  * @name: the playlist name
- * @settings_name: the settings name for the playlist (GSettings path friendly)
+ * @settings: GSettings instance, or NULL to have one created
  * @local: if %TRUE, the playlist is local to the library
  * @entry_type: type of database entries that can be added to the playlist.
  *
@@ -377,25 +353,15 @@ rb_static_playlist_source_constructed (GObject *object)
  * Return value: new playlist.
  */
 RBSource *
-rb_static_playlist_source_new (RBShell *shell, const char *name, const char *settings_name, gboolean local, RhythmDBEntryType *entry_type)
+rb_static_playlist_source_new (RBShell *shell, const char *name, GSettings *settings, gboolean local, RhythmDBEntryType *entry_type)
 {
 	RBSource *source;
-	GSettings *settings;
 	GtkBuilder *builder;
 	GMenu *toolbar;
 
 	if (name == NULL)
 		name = "";
 
-	if (settings_name != NULL) {
-		char *path;
-		path = g_strdup_printf ("/org/gnome/rhythmbox/playlist/%s/", settings_name);
-		settings = g_settings_new_with_path ("org.gnome.rhythmbox.source", path);
-		g_free (path);
-	} else {
-		settings = NULL;
-	}
-
 	builder = rb_builder_load ("playlist-toolbar.ui", NULL);
 	toolbar = G_MENU (gtk_builder_get_object (builder, "playlist-toolbar"));
 	rb_application_link_shared_menus (RB_APPLICATION (g_application_get_default ()), toolbar);
@@ -485,6 +451,7 @@ rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNode
 /**
  * rb_static_playlist_source_new_from_xml:
  * @shell: the #RBShell
+ * @name: playlist name
  * @node: XML node containing playlist entries
  *
  * Constructs a new playlist from the given XML document node.
@@ -492,10 +459,10 @@ rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNode
  * Return value: playlist read from XML
  */
 RBSource *
-rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
+rb_static_playlist_source_new_from_xml (RBShell *shell, const char *name, xmlNodePtr node)
 {
 	RBSource *psource = rb_static_playlist_source_new (shell,
-							   NULL,
+							   name,
 							   NULL,
 							   TRUE,
 							   RHYTHMDB_ENTRY_TYPE_SONG);
