@@ -49,6 +49,7 @@
  * @RB_ENTRY_VIEW_COL_TRACK_NUMBER: the track number column
  * @RB_ENTRY_VIEW_COL_TITLE: the title column
  * @RB_ENTRY_VIEW_COL_ARTIST: the artist column
+ * @RB_ENTRY_VIEW_COL_COMPOSER: the composer column
  * @RB_ENTRY_VIEW_COL_ALBUM: the album column
  * @RB_ENTRY_VIEW_COL_GENRE: the genre column
  * @RB_ENTRY_VIEW_COL_DURATION: the duration column
@@ -91,7 +92,6 @@
 #include "rhythmdb-query-model.h"
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-cell-renderer-rating.h"
-#include "rb-stock-icons.h"
 #include "rb-shell-player.h"
 #include "rb-cut-and-paste-code.h"
 
@@ -153,6 +153,8 @@ static void rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 static void rb_entry_view_pixbuf_clicked_cb (RBEntryView *view,
 					     const char *path,
 					     RBCellRendererPixbuf *cellpixbuf);
+static void rb_entry_view_playing_column_clicked_cb (GtkTreeViewColumn *column,
+						     RBEntryView *view);
 static gboolean rb_entry_view_button_press_cb (GtkTreeView *treeview,
 					      GdkEventButton *event,
 					      RBEntryView *view);
@@ -828,7 +830,11 @@ rb_entry_view_playing_cell_data_func (GtkTreeViewColumn *column,
 	if (entry == view->priv->playing_entry) {
 		switch (view->priv->playing_state) {
 		case RB_ENTRY_VIEW_PLAYING:
-			name = "media-playback-start-symbolic";
+			if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
+				name = "media-playback-start-rtl-symbolic";
+			} else {
+				name = "media-playback-start-symbolic";
+			}
 			break;
 		case RB_ENTRY_VIEW_PAUSED:
 			name = "media-playback-pause-symbolic";
@@ -1314,9 +1320,15 @@ rb_entry_view_get_column (RBEntryView *view, RBEntryViewColumn coltype)
 	case RB_ENTRY_VIEW_COL_LOCATION:
 		propid = RHYTHMDB_PROP_LOCATION;
 		break;
+	case RB_ENTRY_VIEW_COL_BPM:
+		propid = RHYTHMDB_PROP_BPM;
+		break;
 	case RB_ENTRY_VIEW_COL_ERROR:
 		propid = RHYTHMDB_PROP_PLAYBACK_ERROR;
 		break;
+	case RB_ENTRY_VIEW_COL_COMPOSER:
+		propid = RHYTHMDB_PROP_COMPOSER;
+		break;
 	default:
 		g_assert_not_reached ();
 		propid = -1;
@@ -1440,6 +1452,16 @@ rb_entry_view_append_column (RBEntryView *view,
 		key = "Artist";
 		ellipsize = TRUE;
 		break;
+	case RB_ENTRY_VIEW_COL_COMPOSER:
+		propid = RHYTHMDB_PROP_COMPOSER;
+		cell_data->propid = propid;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
+		sort_propid = RHYTHMDB_PROP_COMPOSER_SORT_KEY;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_composer_sort_func;
+		title = _("Composer");
+		key = "Composer";
+		ellipsize = TRUE;
+		break;
 	case RB_ENTRY_VIEW_COL_ALBUM:
 		propid = RHYTHMDB_PROP_ALBUM;
 		cell_data->propid = propid;
@@ -1763,6 +1785,7 @@ static void
 rb_entry_view_constructed (GObject *object)
 {
 	RBEntryView *view;
+	RhythmDBQueryModel *query_model;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_entry_view_parent_class, constructed, object);
 
@@ -1828,7 +1851,10 @@ rb_entry_view_constructed (GObject *object)
 		/* Playing icon column */
 		column = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
 		renderer = rb_cell_renderer_pixbuf_new ();
-		g_object_set (renderer, "stock-size", GTK_ICON_SIZE_MENU, NULL);
+		g_object_set (renderer,
+			      "stock-size", GTK_ICON_SIZE_MENU,
+			      "follow-state", TRUE,
+			      NULL);
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 		gtk_tree_view_column_set_cell_data_func (column, renderer,
 							 (GtkTreeCellDataFunc)
@@ -1836,12 +1862,11 @@ rb_entry_view_constructed (GObject *object)
 							 view,
 							 NULL);
 
-		image_widget = gtk_image_new_from_icon_name ("audio-volume-high", GTK_ICON_SIZE_MENU);
+		image_widget = gtk_image_new_from_icon_name ("audio-volume-high-symbolic", GTK_ICON_SIZE_MENU);
 		gtk_tree_view_column_set_widget (column, image_widget);
-		gtk_widget_show (image_widget);
+		gtk_widget_show_all (image_widget);
 
 		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-		gtk_tree_view_column_set_clickable (column, FALSE);
 		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, NULL);
 		gtk_tree_view_column_set_fixed_width (column, width + 5);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
@@ -1852,14 +1877,17 @@ rb_entry_view_constructed (GObject *object)
 
 		gtk_widget_set_tooltip_text (gtk_tree_view_column_get_widget (column),
 					     _("Now Playing"));
+
+		g_signal_connect (column,
+				  "clicked",
+				  G_CALLBACK (rb_entry_view_playing_column_clicked_cb),
+				  view);
+		gtk_tree_view_column_set_clickable (column, TRUE);
 	}
 
-	{
-		RhythmDBQueryModel *query_model;
-		query_model = rhythmdb_query_model_new_empty (view->priv->db);
-		rb_entry_view_set_model (view, RHYTHMDB_QUERY_MODEL (query_model));
-		g_object_unref (query_model);
-	}
+	query_model = rhythmdb_query_model_new_empty (view->priv->db);
+	rb_entry_view_set_model (view, RHYTHMDB_QUERY_MODEL (query_model));
+	g_object_unref (query_model);
 }
 
 static void
@@ -1913,6 +1941,15 @@ rb_entry_view_pixbuf_clicked_cb (RBEntryView          *view,
 	rhythmdb_entry_unref (entry);
 }
 
+static void
+rb_entry_view_playing_column_clicked_cb (GtkTreeViewColumn *column,
+					 RBEntryView *view)
+{
+	if (view->priv->playing_entry) {
+		rb_entry_view_scroll_to_entry (view, view->priv->playing_entry);
+	}
+}
+
 static void
 rb_entry_view_playing_song_changed (RBShellPlayer *player,
 				    RhythmDBEntry *entry,
@@ -2593,6 +2630,7 @@ rb_entry_view_column_get_type (void)
 			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LOCATION, "location"),
 			ENUM_ENTRY (RB_ENTRY_VIEW_COL_BPM, "bpm"),
 			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ERROR, "error"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_COMPOSER, "composer"),
 			{ 0, 0, 0 }
 		};
 
